@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001  John Elliott <jce@seasip.demon.co.uk>            *
+ *    Copyright (C) 2001  John Elliott <seasip.webmaster@gmail.com>            *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -53,7 +53,8 @@
 DRV_CLASS dc_linux = 
 {
 	sizeof(LINUX_DSK_DRIVER),
-	"floppy",
+	NULL,		/* superclass */
+	"floppy\0",
 	"Linux floppy driver",
 	&linux_open,	/* open */
 	&linux_creat,	/* create new */
@@ -235,9 +236,10 @@ dsk_err_t linux_read(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
                              void *buf, dsk_pcyl_t cylinder,
                               dsk_phead_t head, dsk_psect_t sector)
 {
+/* Don't dg_x_sector() here; if required it will have been done in dg_ls2ps() */
 	return linux_xread(self, geom, buf, cylinder, head, cylinder, 
 			dg_x_head(geom, head), 
-			dg_x_sector(geom, head, sector), geom->dg_secsize, 0);
+			sector, geom->dg_secsize, 0);
 }
 
 dsk_err_t linux_xread(DSK_DRIVER *self, const DSK_GEOMETRY *geom, void *buf,
@@ -259,7 +261,7 @@ dsk_err_t linux_xread(DSK_DRIVER *self, const DSK_GEOMETRY *geom, void *buf,
 	if (err) return err;
 
 	if (geom->dg_noskip)  mask &= ~0x20;	/* Don't skip deleted data */
-	if (geom->dg_fm)      mask &= ~0x40;	/* FM recording mode */
+	if (geom->dg_fm & RECMODE_MASK) mask &= ~0x40;	/* FM recording mode */
 	if (geom->dg_nomulti) mask &= ~0x80;	/* Disable multitrack */
 
 	init_raw_cmd(&raw_cmd);
@@ -298,9 +300,10 @@ dsk_err_t linux_write(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
                              const void *buf, dsk_pcyl_t cylinder,
                               dsk_phead_t head, dsk_psect_t sector)
 {
+/* Don't dg_x_sector() here; if required it will have been done in dg_ls2ps() */
 	return linux_xwrite(self, geom, buf, cylinder, head, cylinder, 
 			dg_x_head(geom, head),
-			dg_x_sector(geom, head, sector), 
+			sector, 
 			geom->dg_secsize, 0);
 }
 
@@ -324,7 +327,7 @@ dsk_err_t linux_xwrite(DSK_DRIVER *self, const DSK_GEOMETRY *geom, const void *b
 	if (err) return err;
 
 	if (geom->dg_noskip)  mask &= ~0x20;	/* Don't skip deleted data */
-	if (geom->dg_fm)      mask &= ~0x40;
+	if (geom->dg_fm & RECMODE_MASK)      mask &= ~0x40;
 	if (geom->dg_nomulti) mask &= ~0x80;
 
 	init_raw_cmd(&raw_cmd);
@@ -376,7 +379,7 @@ dsk_err_t linux_format(DSK_DRIVER *self, DSK_GEOMETRY *geom,
 
 	err = check_geom(lxself, geom);
 	if (err) return err;
-	if (geom->dg_fm)      mask &= ~0x40;
+	if (geom->dg_fm & RECMODE_MASK)      mask &= ~0x40;
 	if (geom->dg_nomulti) mask &= ~0x80;
 
 	buf = dsk_malloc(geom->dg_sectors * 4);
@@ -433,7 +436,7 @@ dsk_err_t linux_secid(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	lxself = (LINUX_DSK_DRIVER *)self;
 	if (lxself->lx_fd < 0) return DSK_ERR_NOTRDY;
 
-	if (geom->dg_fm)      mask &= ~0x40;
+	if (geom->dg_fm & RECMODE_MASK)      mask &= ~0x40;
 	if (geom->dg_nomulti) mask &= ~0x80;
 
 /* [v0.8.3] It was necessary to add this check correctly to detect 100k 
@@ -457,7 +460,7 @@ dsk_err_t linux_secid(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	result->fmt_cylinder = raw_cmd.reply[3];	
 	result->fmt_head     = raw_cmd.reply[4];	
 	result->fmt_sector   = raw_cmd.reply[5];	
-	result->fmt_secsize  = 128 << raw_cmd.reply[6];	
+	result->fmt_secsize  = dsk_expand_psh(raw_cmd.reply[6]);
 	lxself->lx_cylinder = cylinder;
 	return DSK_ERR_OK;
 }
@@ -541,7 +544,7 @@ dsk_err_t linux_xtread(DSK_DRIVER *self, const DSK_GEOMETRY *geom, void *buf,
 	err = check_geom(lxself, geom);
 	if (err) return err;
 
-	if (geom->dg_fm)      mask &= ~0x40;
+	if (geom->dg_fm & RECMODE_MASK)      mask &= ~0x40;
 	if (geom->dg_nomulti) mask &= ~0x80;
 
 	init_raw_cmd(&raw_cmd);
