@@ -24,6 +24,7 @@
  * access to a flat file with the tracks laid out in the SIDES_ALT order */
 
 #include <stdio.h>
+#include <assert.h>
 #include "libdsk.h"
 #include "drvi.h"
 #include "ldbs.h"
@@ -215,11 +216,11 @@ dsk_err_t posix_close(DSK_DRIVER *self)
 }
 
 
-long posix_offset(POSIX_DSK_DRIVER *pxself, const DSK_GEOMETRY *geom,
+unsigned long posix_offset(POSIX_DSK_DRIVER *pxself, const DSK_GEOMETRY *geom,
 			dsk_pcyl_t cylinder, dsk_phead_t head, 
 			dsk_psect_t sector)
 {
-	long offset = 0;
+	unsigned long offset = 0;
 
 	/* Work out the offset based on the sidedness of the disk image
 	 * (not the sidedness of the geometry) */
@@ -255,7 +256,7 @@ dsk_err_t posix_read(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
                               dsk_phead_t head, dsk_psect_t sector)
 {
 	POSIX_DSK_DRIVER *pxself;
-	long offset;
+	unsigned long offset;
 
 	if (!buf || !self || !geom) return DSK_ERR_BADPTR;
 	CHECK_CLASS(self);
@@ -300,11 +301,14 @@ dsk_err_t posix_write(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	unsigned long offset;
 	dsk_err_t err;
 
-	if (!buf || !self || !geom)
+	if (!buf || !self || !geom) return DSK_ERR_BADPTR;
+
 	CHECK_CLASS(self);
 
 	if (!pxself->px_fp) return DSK_ERR_NOTRDY;
 	if (pxself->px_readonly) return DSK_ERR_RDONLY;
+	if (sector < geom->dg_secbase || sector >= geom->dg_secbase + geom->dg_sectors)
+		return DSK_ERR_NOADDR;
 
 	offset = posix_offset(pxself, geom, cylinder, head, sector);
 
@@ -354,7 +358,7 @@ dsk_err_t posix_format(DSK_DRIVER *self, DSK_GEOMETRY *geom,
 	if (pxself->px_filesize < offset + trklen)
 		pxself->px_filesize = offset + trklen;
 
-	while (trklen--) 
+	for (++trklen; trklen > 1; trklen--)
 		if (fputc(filler, pxself->px_fp) == EOF) return DSK_ERR_SYSERR;	
 
 	return DSK_ERR_OK;
@@ -477,6 +481,7 @@ dsk_err_t posix_to_ldbs(DSK_DRIVER *self, struct ldbs **result, DSK_GEOMETRY *ge
 			th->sector[sec].id_head = head;
 			th->sector[sec].id_sec  = sec + geom->dg_secbase;
 			th->sector[sec].id_psh  = dsk_get_psh(geom->dg_secsize);
+			th->sector[sec].datalen = geom->dg_secsize;
 			th->sector[sec].copies = 0;
 			for (n = 1; n < (int)(geom->dg_secsize); n++)
 			{
@@ -593,7 +598,8 @@ static dsk_err_t posix_from_ldbs_callback(PLDBS ldbs, dsk_pcyl_t cyl,
 	else
 	{
 		/* Otherwise just regurgitate the whole track in one hit */
-		err = ldbs_load_track(ldbs, th, (void *)&secbuf, &len, 0);
+		err = ldbs_load_track(ldbs, th, (void *)&secbuf, &len, 0,
+				LLTO_DATA_ONLY);
 		if (err) return err;
 		if (fwrite(secbuf, 1, len, pxself->px_fp) < len)
 		{
