@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001-2, 2005  John Elliott <jce@seasip.demon.co.uk>    *
+ *    Copyright (C) 2001-2, 2005  John Elliott <seasip.webmaster@gmail.com>    *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -39,44 +39,48 @@ typedef struct dsk_namedgeom
 static DSK_NAMEDGEOM stdg[] = 
 {
         /*    sidedness cyl hd sec  psn  sz   rate    rwgap  fmtgap  fm  nomulti*/
-{"pcw180",  { SIDES_ALT,     40, 1, 9,    1, 512, RATE_SD, 0x2A, 0x52,   0,  0 }, "PCW / IBM 180k" }, /* 180k */
-{"cpcsys",  { SIDES_ALT,     40, 1, 9, 0x41, 512, RATE_SD, 0x2A, 0x52,   0,  0 }, "CPC System" }, /* CPC system */
-{"cpcdata", { SIDES_ALT,     40, 1, 9, 0xC1, 512, RATE_SD, 0x2A, 0x52,   0,  0 }, "CPC Data" }, /* CPC data */
-{"pcw720",  { SIDES_ALT,     80, 2, 9,    1, 512, RATE_SD, 0x2A, 0x52,   0,  0 }, "PCW / IBM 720k" }, /* 720k */
-{"pcw1440", { SIDES_ALT,     80, 2,18,    1, 512, RATE_HD, 0x1B, 0x54,   0,  0 }, "PcW16 / IBM 1440k "}, /* 1.4M */
-{"ibm160",  { SIDES_ALT,     40, 1, 8,    1, 512, RATE_SD, 0x2A, 0x50,   0,  0 }, "IBM 160k (CP/M-86 / DOSPLUS)" }, /* 160k */
-{"ibm320",  { SIDES_ALT,     40, 2, 8,    1, 512, RATE_SD, 0x2A, 0x50,   0,  0 }, "IBM 320k (CP/M-86 / DOSPLUS)" }, /* 320k */
-{"ibm360",  { SIDES_ALT,     40, 2, 9,    1, 512, RATE_SD, 0x2A, 0x52,   0,  0 }, "IBM 360k (CP/M-86 / DOSPLUS)" }, /* 360k */
-{"ibm720",  { SIDES_OUTBACK, 80, 2, 9,    1, 512, RATE_SD, 0x2A, 0x52,   0,  0 }, "IBM 720k (144FEAT)", }, /* 720k 144FEAT */
-{"ibm1200", { SIDES_OUTBACK, 80, 2,15,    1, 512, RATE_HD, 0x1B, 0x54,   0,  0 }, "IBM 1.2M (144FEAT)", }, /* 1.2M 144FEAT */
-{"ibm1440", { SIDES_OUTBACK, 80, 2,18,    1, 512, RATE_HD, 0x1B, 0x54,   0,  0 }, "IBM 1.4M (144FEAT)", }, /* 1.4M 144FEAT */
-{"acorn160",    { SIDES_OUTOUT,  40, 1,16,    0, 256, RATE_SD, 0x12, 0x60,   0,  0 }, "Acorn 160k" }, /* Acorn 160k */
-{"acorn320",    { SIDES_OUTOUT,  80, 1,16,    0, 256, RATE_SD, 0x12, 0x60,   0,  0 }, "Acorn 320k" }, /* Acorn 320k */
-{"acorn640",    { SIDES_OUTOUT,  80, 2,16,    0, 256, RATE_SD, 0x12, 0x60,   0,  0 }, "Acorn 640k" }, /* Acorn 640k */
-{"acorn800",    { SIDES_ALT,     80, 2, 5,    0,1024, RATE_SD, 0x04, 0x05,   0,  0 }, "Acorn 800k" }, /* Acorn 800k */
-{"acorn1600",   { SIDES_ALT,     80, 2,10,    0,1024, RATE_HD, 0x04, 0x05,   0,  0 }, "Acorn 1600k" }, /* Acorn 1600k */
-{"pcw800",  { SIDES_ALT,     80, 2,10,    1, 512, RATE_SD, 0x0C, 0x17,   0,  0 }, "PCW 800k" }, /* 800k */
-{"pcw200",  { SIDES_ALT,     40, 1,10,    1, 512, RATE_SD, 0x0C, 0x17,   0,  0 }, "PCW 200k" }, /* 200k */
+{"pcw180",  { SIDES_ALT,     40, 1, 9,    1, 512, RATE_SD, 0x2A, 0x52,   RECMODE_MFM,  0 }, "PCW / IBM 180k" }, /* 180k */
+{"cpcsys",  { SIDES_ALT,     40, 1, 9, 0x41, 512, RATE_SD, 0x2A, 0x52,   RECMODE_MFM,  0 }, "CPC System" }, /* CPC system */
+{"cpcdata", { SIDES_ALT,     40, 1, 9, 0xC1, 512, RATE_SD, 0x2A, 0x52,   RECMODE_MFM,  0 }, "CPC Data" }, /* CPC data */
+{"pcw720",  { SIDES_ALT,     80, 2, 9,    1, 512, RATE_SD, 0x2A, 0x52,   RECMODE_MFM,  0 }, "PCW / IBM 720k" }, /* 720k */
+{"pcw1440", { SIDES_ALT,     80, 2,18,    1, 512, RATE_HD, 0x1B, 0x54,   RECMODE_MFM,  0 }, "PcW16 / IBM 1440k "}, /* 1.4M */
+{"ibm160",  { SIDES_ALT,     40, 1, 8,    1, 512, RATE_SD, 0x2A, 0x50,   RECMODE_MFM,  0 }, "IBM 160k (CP/M-86 / DOSPLUS)" }, /* 160k */
+{"ibm320",  { SIDES_ALT,     40, 2, 8,    1, 512, RATE_SD, 0x2A, 0x50,   RECMODE_MFM,  0 }, "IBM 320k (CP/M-86 / DOSPLUS)" }, /* 320k */
+{"ibm360",  { SIDES_ALT,     40, 2, 9,    1, 512, RATE_SD, 0x2A, 0x52,   RECMODE_MFM,  0 }, "IBM 360k (CP/M-86 / DOSPLUS)" }, /* 360k */
+{"ibm720",  { SIDES_OUTBACK, 80, 2, 9,    1, 512, RATE_SD, 0x2A, 0x52,   RECMODE_MFM,  0 }, "IBM 720k (144FEAT)", }, /* 720k 144FEAT */
+{"ibm1200", { SIDES_OUTBACK, 80, 2,15,    1, 512, RATE_HD, 0x1B, 0x54,   RECMODE_MFM,  0 }, "IBM 1.2M (144FEAT)", }, /* 1.2M 144FEAT */
+{"ibm1440", { SIDES_OUTBACK, 80, 2,18,    1, 512, RATE_HD, 0x1B, 0x54,   RECMODE_MFM,  0 }, "IBM 1.4M (144FEAT)", }, /* 1.4M 144FEAT */
+{"acorn160",    { SIDES_OUTOUT,  40, 1,16,    0, 256, RATE_SD, 0x12, 0x60,   RECMODE_MFM,  0 }, "Acorn 160k" }, /* Acorn 160k */
+{"acorn320",    { SIDES_OUTOUT,  80, 1,16,    0, 256, RATE_SD, 0x12, 0x60,   RECMODE_MFM,  0 }, "Acorn 320k" }, /* Acorn 320k */
+{"acorn640",    { SIDES_OUTOUT,  80, 2,16,    0, 256, RATE_SD, 0x12, 0x60,   RECMODE_MFM,  0 }, "Acorn 640k" }, /* Acorn 640k */
+{"acorn800",    { SIDES_ALT,     80, 2, 5,    0,1024, RATE_SD, 0x04, 0x05,   RECMODE_MFM,  0 }, "Acorn 800k" }, /* Acorn 800k */
+{"acorn1600",   { SIDES_ALT,     80, 2,10,    0,1024, RATE_HD, 0x04, 0x05,   RECMODE_MFM,  0 }, "Acorn 1600k" }, /* Acorn 1600k */
+{"pcw800",  { SIDES_ALT,     80, 2,10,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_MFM,  0 }, "PCW 800k" }, /* 800k */
+{"pcw200",  { SIDES_ALT,     40, 1,10,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_MFM,  0 }, "PCW 200k" }, /* 200k */
 {"bbc100",  { SIDES_ALT,     40, 1,10,    0, 256, RATE_SD, 0x2A, 0x50,   1,  0 }, "BBC 100k" }, /* 100k */
 {"bbc200",  { SIDES_ALT,     80, 1,10,    0, 256, RATE_SD, 0x2A, 0x50,   1,  0 }, "BBC 200k" }, /* 200k */
-{"mbee400", { SIDES_ALT,     40, 1,10,    0, 512, RATE_SD, 0x0C, 0x17,   0,  0 }, "Microbee 400k" }, /* 400k */
-{"mgt800",  { SIDES_OUTOUT,  80, 2,10,    1, 512, RATE_SD, 0x0C, 0x17,   0,  0 }, "MGT 800k" }, /* MGT 800k */
-{"trdos640",{ SIDES_ALT,     80, 2,16,    1, 256, RATE_SD, 0x12, 0x60,   0,  0 }, "TR-DOS 640k" }, /* TR-DOS 640k */
-{"ampro200",{ SIDES_ALT,     40, 1,10,   1, 512, RATE_SD, 0x0C, 0x17,   0,  0 }, "Ampro 40 track single-sided" }, /* Ampro 200k (22DISK AMP1) */
-{"ampro400d",{ SIDES_ALT,     40, 2,10,   17, 512, RATE_SD, 0x0C, 0x17,   0,  0 }, "Ampro 40 track double-sided" }, /* Ampro 400k (22DISK AMP2) */
-{"ampro400s",{ SIDES_ALT,     80, 1, 5,   1,1024, RATE_SD, 0x04, 0x05,   0,  0 }, "Ampro 80 track single-sided" }, /* Ampro 400k (22DISK AMP3) */
-{"ampro800",{ SIDES_ALT,     80, 2, 5,   17,1024, RATE_SD, 0x04, 0x05,   0,  0 }, "Ampro 80 track double-sided" }, /* Ampro 800k (22DISK AMP4) */
+{"mbee400", { SIDES_ALT,     40, 1,10,    0, 512, RATE_SD, 0x0C, 0x17,   RECMODE_MFM,  0 }, "Microbee 400k" }, /* 400k */
+{"mgt800",  { SIDES_OUTOUT,  80, 2,10,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_MFM,  0 }, "MGT 800k" }, /* MGT 800k */
+{"trdos640",{ SIDES_ALT,     80, 2,16,    1, 256, RATE_SD, 0x12, 0x60,   RECMODE_MFM,  0 }, "TR-DOS 640k" }, /* TR-DOS 640k */
+{"ampro200",{ SIDES_ALT,     40, 1,10,   1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_MFM,  0 }, "Ampro 40 track single-sided" }, /* Ampro 200k (22DISK AMP1) */
+{"ampro400d",{ SIDES_ALT,     40, 2,10,   17, 512, RATE_SD, 0x0C, 0x17,   RECMODE_MFM,  0 }, "Ampro 40 track double-sided" }, /* Ampro 400k (22DISK AMP2) */
+{"ampro400s",{ SIDES_ALT,     80, 1, 5,   1,1024, RATE_SD, 0x04, 0x05,   RECMODE_MFM,  0 }, "Ampro 80 track single-sided" }, /* Ampro 400k (22DISK AMP3) */
+{"ampro800",{ SIDES_ALT,     80, 2, 5,   17,1024, RATE_SD, 0x04, 0x05,   RECMODE_MFM,  0 }, "Ampro 80 track double-sided" }, /* Ampro 800k (22DISK AMP4) */
+{"pcw1200", { SIDES_ALT,     80, 2,15,    1, 512, RATE_HD, 0x1B, 0x54,   RECMODE_MFM,  0 }, "PcW16 / IBM 1200k "}, /* 1.2M */
+{"mac400",  { SIDES_ALT,     80, 1,12,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_GCR_MAC, 0 }, "Macintosh GCR 400k"}, /* Mac GCR 400k */
+{"mac800",  { SIDES_ALT,     80, 2,12,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_GCR_MAC, 0 }, "Macintosh GCR 800k"}, /* Mac GCR 800k */
+
 /* Geometries below this line don't appear in dsk_format_t and can be accessed
  * only by name. */
 
-{"myz80",   { SIDES_ALT,     64, 1,128,   0,1024, RATE_ED, 0x2A, 0x52,   0,  0 }, "MYZ80 8Mb" }, /* MYZ80 8Mb */
+{"myz80",   { SIDES_ALT,     64, 1,128,   0,1024, RATE_ED, 0x2A, 0x52,   RECMODE_MFM,  0 }, "MYZ80 8Mb" }, /* MYZ80 8Mb */
 
 /* re this comment ('This was commented out in libdsk-1.1.3, but I can't 
  * remember why. Bring it back.') -- I think I now remember why. The 
  * numeric format IDs were out of sync with the actual definitions, and I
  * think this may be part of the reason. Move it down to the bottom, beyond
  * the last numeric ID. */
-{"pcpm320", { SIDES_OUTBACK, 40, 2, 8,    1, 512, RATE_SD, 0x2A, 0x50,   0,  0 }, "IBM 320k (CP/M-86 / DOSPLUS)" }, /* 320k */
+{"pcpm320", { SIDES_OUTBACK, 40, 2, 8,    1, 512, RATE_SD, 0x2A, 0x50,   RECMODE_MFM,  0 }, "IBM 320k (CP/M-86 / DOSPLUS)" }, /* 320k */
 };
 
 
@@ -340,8 +344,40 @@ dsk_err_t dg_parseline(char *linebuf, DSK_GEOMETRY *dg, char *description)
     if (!strcmp(linebuf, "fm"))
     {
         for (s = value; s[0]; s++) *s = tolower(*s);
-        if (!strcmp(value, "y")) dg->dg_fm = 1;
-        if (!strcmp(value, "n")) dg->dg_fm = 0;
+        if (!strcmp(value, "y")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_FM;
+        if (!strcmp(value, "n")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_MFM;
+    }
+    /* [1.4.1] Allow 'recmode=mfm' / 'recmode=fm' as a synonym for 'fm=y / fm=n'
+     *         Obviously this syntax would allow additional recording modes in
+     *         future. */
+    if (!strcmp(linebuf, "recmode"))
+    {
+	int xrecmode;
+
+        for (s = value; s[0]; s++) *s = tolower(*s);
+        if (!strcmp(value, "fm")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_FM;
+        if (!strcmp(value, "mfm")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_MFM;
+        if (!strcmp(value, "gcr_mac")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_GCR_MAC;
+        if (!strcmp(value, "gcr_prodos")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_GCR_PRODOS;
+        if (!strcmp(value, "gcr_lisa")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_GCR_LISA;
+        if (!strncmp(value, "gcr_mac_", 8) && sscanf(value + 8, "%d", &xrecmode)) 
+	    dg->dg_fm = ((dg->dg_fm & RECMODE_FLAGMASK) | (xrecmode & 0x1F) + RECMODE_GCR_FIRST);
+    }
+    /* [1.4.1] 'Complement' flag */
+    if (!strcmp(linebuf, "complement"))
+    {
+        for (s = value; s[0]; s++) *s = tolower(*s);
+        if (!strcmp(value, "y")) 
+	    dg->dg_fm |= RECMODE_COMPLEMENT;
+        if (!strcmp(value, "n")) 
+	    dg->dg_fm &= ~RECMODE_COMPLEMENT;
     }
     if (!strcmp(linebuf, "multitrack"))
     {
@@ -362,6 +398,8 @@ dsk_err_t dg_parseline(char *linebuf, DSK_GEOMETRY *dg, char *description)
 
 dsk_err_t dg_store(FILE *fp, DSK_GEOMETRY *dg, char *description)
 {
+    int recmode;
+
     if (description) fprintf(fp, "description=%s\n", description);
     switch(dg->dg_sidedness)
     {
@@ -384,7 +422,24 @@ dsk_err_t dg_store(FILE *fp, DSK_GEOMETRY *dg, char *description)
     }
     fprintf(fp, "rwgap=%d\n", dg->dg_rwgap);
     fprintf(fp, "fmtgap=%d\n", dg->dg_fmtgap);
-    fprintf(fp, "fm=%c\n", dg->dg_fm ? 'Y' : 'N');
+  
+    recmode = dg->dg_fm & RECMODE_MASK;
+    switch (recmode)
+    {
+        case RECMODE_MFM:        fprintf(fp, "recmode=MFM\n"); break;
+        case RECMODE_FM:         fprintf(fp, "recmode=FM\n"); break;
+        case RECMODE_GCR_MAC:    fprintf(fp, "recmode=GCR_MAC\n"); break;
+        case RECMODE_GCR_LISA:   fprintf(fp, "recmode=GCR_LISA\n"); break;
+        case RECMODE_GCR_PRODOS: fprintf(fp, "recmode=GCR_PRODOS\n"); break;
+	default: 
+		if (recmode >= RECMODE_GCR_FIRST && recmode <= RECMODE_GCR_LAST)
+		{
+			fprintf(fp, "recmode=GCR_MAC_%d\n",
+				(recmode - RECMODE_GCR_FIRST) & 0x1F);
+		}
+		break;
+    }
+    fprintf(fp, "complement=%c\n", (dg->dg_fm & RECMODE_COMPLEMENT) ? 'Y' : 'N');
     fprintf(fp, "multitrack=%c\n", dg->dg_nomulti ? 'N' : 'Y');
     fprintf(fp, "skipdeleted=%c\n", dg->dg_noskip ? 'N' : 'Y');
     return DSK_ERR_OK;
