@@ -31,7 +31,8 @@
 DRV_CLASS dc_remote =
 {
 	sizeof(DSK_DRIVER),	/* [deliberately] No extra data */
-	"remote",
+	NULL,		/* superclass */
+	"remote\0",
 	"Remote LibDsk instance",
 	remote_open,
 	remote_creat,
