@@ -65,7 +65,7 @@ int help(int argc, char **argv)
 
 static void report(const char *s)
 {
-	fprintf(stderr,"%s\r", s);
+	fprintf(stderr,"%-79.79s\r", s);
 	fflush(stderr);
 }
 
@@ -136,10 +136,22 @@ const char *show_rate(dsk_rate_t r)
 
 const char *show_recmode(int recmode)
 {
-	switch (recmode & RECMODE_MASK)
+	static char buf[50];
+
+	recmode &= RECMODE_MASK;
+
+	switch (recmode)
 	{
 		case RECMODE_MFM: return "MFM";
 		case RECMODE_FM:  return "FM";
+		case RECMODE_GCR_MAC: return "GCR (Macintosh)";
+		case RECMODE_GCR_LISA: return "GCR (Lisa)";
+		case RECMODE_GCR_PRODOS: return "GCR (Apple II Prodos)";
+	}
+	if (recmode >= RECMODE_GCR_FIRST && recmode <= RECMODE_GCR_LAST)
+	{
+		sprintf(buf, "GCR (Apple %d)", recmode - RECMODE_GCR_FIRST);
+		return buf;
 	}
 	return "???";
 }
@@ -251,7 +263,10 @@ int do_login(int argc, char *outfile, char *outtyp, char *outcomp, int forcehead
 			++opt;
 		}
 	}
-	if (outdr) dsk_close(&outdr);
+	if (outdr) 
+	{
+		if (!e) e = dsk_close(&outdr); else dsk_close(&outdr);
+	}
 	if (e)
 	{
 		fprintf(stderr, "%s: %s\n", outfile, dsk_strerror(e));
