@@ -245,9 +245,15 @@ int do_copy(char *infile, char *outfile)
 		    {
                         if (md3)
                         {
-			// MD3 discs have 256-byte sectors in cyl 1 head 1
-			    if (cyl == 1 && head == 1) dg.dg_secsize = 256;
-			    else		       dg.dg_secsize = 512;
+			    // MD3 discs have 256-byte sectors on their
+			    // fourth track
+			    dsk_ltrack_t ltrack;
+
+			    e = dg_pt2lt(&dg, cyl, head, &ltrack);
+			    if (e == DSK_ERR_OK && ltrack == 3)
+			    { dg.dg_secsize = 256; }
+			    else
+			    { dg.dg_secsize = 512; }
                         }
 			// Format track!
                         if (e == DSK_ERR_OK && (!noformat)) 
